// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.util;


import static org.junit.Assert.assertTrue;

import com.amazonaws.services.sqs.model.Message;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang3.StringUtils;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AwsTestUtils extends TestUtils {
    AwsCognitoClient client = new AwsCognitoClient();
    private static final ObjectMapper objectMapper = new ObjectMapper();

      @Override
    public String getOpsToken() throws Exception {
        if (opsToken==null || StringUtils.isEmpty(opsToken) ) {
            opsToken= client.getToken(AwsConfig.getAWSCognitoUser(),AwsConfig.getAWSCognitoPassword(),"bearer");
        }
        return  opsToken;
    }

    public DpsHeaders getDpsHeaders() throws Exception {
        Map<String, String> headers = new HashMap<>();
        headers.put(DpsHeaders.DATA_PARTITION_ID, TestUtils.getOsduTenant());
        headers.put(DpsHeaders.AUTHORIZATION, this.getOpsToken());
        //hardcoding user here for 200 response tests. This is just initializing the subscription creation
        headers.put(DpsHeaders.USER_ID, AwsConfig.getAWSCognitoUser());

        return DpsHeaders.createFromMap(headers);
    }

    public static <T> T unwrapFirstMessage(Message message, Class<T[]> classType) throws Exception {
        T[] submessages = objectMapper.readValue(message.getBody(), classType);
        assertTrue("Must have at least 1 sub-message!", 1 <= submessages.length);
        return submessages[0];
    }

    public static <T> T unwrapFirst(List<Message> messages, Class<T[]> classType) throws Exception {
        assertTrue("Must have at least 1 message!", 1 <= messages.size());

        Message firstMessage = messages.get(0);
        return unwrapFirstMessage(firstMessage, classType);
    }

    public static <T> T unwrapSingleFirst(List<Message> messages, Class<T> classType) throws Exception {
        assertTrue("Must have at least 1 message!", 1 <= messages.size());

        Message firstMessage = messages.get(0);
        return objectMapper.readValue(firstMessage.getBody(), classType);
    }

    //These users don't have access to the API, so getting the token for NoAccessUser
    @Override
    public String getAdminToken() throws Exception {
        if (adminToken==null || StringUtils.isEmpty(adminToken)) {
            adminToken=client.getToken(AwsConfig.getAWSCognitoNoAccessUser(),AwsConfig.getAWSCognitoPassword(),"bearer");
        }
        return adminToken;
    }

    //These users don't have access to the API, so getting the token for NoAccessUser
    @Override
    public String getEditorToken() throws Exception {
        if (editorToken==null || StringUtils.isEmpty(editorToken)) {
            editorToken=client.getToken(AwsConfig.getAWSCognitoNoAccessUser(),AwsConfig.getAWSCognitoPassword(),"bearer");
        }
        return editorToken;
    }

    //These users don't have access to the API, so getting the token for NoAccessUser
    @Override
    public String getNoAccessToken() throws Exception {
        if (noAccessToken==null || StringUtils.isEmpty(noAccessToken)) {
            noAccessToken=client.getToken(AwsConfig.getAWSCognitoNoAccessUser(),AwsConfig.getAWSCognitoPassword(),"bearer");
        }
        return  noAccessToken;
    }
}
